#include "global.h"		// include our global settings

#include <avr/io.h>		// include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include interrupt support
#include <util/crc16.h>
#include <avr/delay.h>
#include "dbimage.h"
#include "emotion_engine.h"
#include "uart.h"		// include uart function library
//#include "rprintf.h"	// include printf function library
//#include "timer.h"		// include timer function library (timing, PWM, etc)
//#include "vt100.h"		// include VT100 terminal support
//#include "encoder.h"	// include encoder driver

#include "../common.h"

static int8_t trame[12];
/*---------- internal ---------*/
static uint8_t crc_run(void);
static void out_init(void);
static void out_trame_create(uint8_t screen_id, uint16_t image_id);
static void out_send(void);
static void send_image(uint8_t cmd, uint8_t cnt, uint8_t screen_id);
static void delayms(uint16_t millis);
/*---------------------------------------------*/

int main(void)
{
  uint8_t cnt=0;
  uint8_t cmd=0;
  out_init();
  DDRD = 1 << PD3;/*Led*/
  while(1){
    if( (PIND & 1<<PD2) == 0){ /*Button press*/
      delayms(50);
      cmd++;
      cnt=0;
      PORTD= PORTD ^ (1 << PD3); /*green Led state  change*/
    }
    if(cmd == sizeof(clip)/sizeof(struct clip_t)){
      cmd = 0;
    }

    send_image(cmd,cnt,SCREEN_G);
    send_image(cmd,cnt,SCREEN_D);
    cnt++;
  }
  
  return 0;
}

/*--------------- out ------------------------------*/
static void delayms(uint16_t millis) {
  uint16_t loop;
  while ( millis ) {
    _delay_ms(1);
    millis--;
  }
}

static void out_init(void)
{
  uartInit();
  uartSetBaudRate(2400); // 10 images/s max
}

static void out_trame_create(uint8_t screen_id, uint16_t image_id)
{
  trame[0] = 0xA5;
  trame[1] = screen_id;

  memcpy_P(&(trame[2]),&(dbimage[image_id][0]),8);
  trame[10] = crc_run();
}

static void out_send(void)
{
  while(uartSendBuffer(trame,sizeof(trame))==FALSE);
}

static uint8_t crc_run()
{
  uint8_t crc = 0, i;
  
  for (i = 0; i < (sizeof (trame)-1); i++)
    crc = _crc_ibutton_update(crc, trame[i]);
  
  return crc;
}

static uint16_t find_image(uint8_t cmd, uint8_t cnt, uint8_t screen)
{
  const uint16_t start_index =   pgm_read_word (&(clip[cmd].start_index));
  const uint8_t size =   pgm_read_byte(&(clip[cmd].size));
  if (cnt > size){
    cnt = size;
  }
 
  return  pgm_read_byte(&(dbvideo[start_index+cnt][screen]));
}

static void send_image(uint8_t cmd, uint8_t cnt, uint8_t screen_id)
{
  uint16_t image_id =  find_image(cmd, cnt, screen_id);
  out_trame_create(screen_id, image_id);
  out_send();
}
